@@ -21,7 +21,7 @@
 #include <config.h>
 #include "query-env.h"
 #include "query-exec.h"
-#include "datadisplay-common.h"
+#include "data-entry.h"
 #include "marshal.h"
 #include "data-grid.h"
 #include "data-form.h"
@@ -54,8 +54,7 @@ struct _MissingDataNode
 {
 	QueryField        *qf;    /* represented QueryField */
 	gchar             *descr; 
-	DataDisplayFns    *fns;	  /* to get the GdaValue from the widget */
-	GtkWidget         *widget;/* real widget */
+	GtkWidget         *data_entry; /* DataEntry if we use a DataEntry */
 };
 
 #define MISSING_DATA_NODE(nde) ((MissingDataNode *) nde)
@@ -69,7 +68,6 @@ struct _ExecUi
 #define EXEC_UI(st) ((ExecUi *) st)
 
 static void add_managed_ui (QueryExec *qx, ExecUi *eui);
-static void managed_ui_destroyed (QueryExec *qx, GtkWidget *wid);
 
 
 /*
@@ -197,21 +195,21 @@ prepare_query (QueryExec * qx)
 	list = qx->priv->q->fields;
 	while (list) {
 		qf = QUERY_FIELD (list->data);
-		if (qf->field_type == QUERY_FIELD_ALLFIELDS) {
+		if (query_field_get_ftype (qf) == QUERY_FIELD_ALLFIELDS) {
 			GSList *nextone = g_slist_next (list);
 			QueryView *view = query_field_allfields_get_queryview (qf);
 
-			if (view && IS_DB_TABLE (view->obj)) {
+			if (view && IS_DB_TABLE (query_view_get_view_obj (view))) {
 				DbTable *table;
 				QueryField *nqf;
 				GSList *fields;
 
-				table = DB_TABLE (view->obj);
+				table = DB_TABLE (query_view_get_view_obj (view));
 				fields = table->fields;
 				while (fields) {
-					nqf = QUERY_FIELD (query_field_new (qx->priv->q, DB_FIELD (fields->data)->name, 
-									    QUERY_FIELD_FIELD));
-					query_field_field_set_field (nqf, view, DB_FIELD (fields->data));
+					nqf = query_field_field_dbfield_new (qx->priv->q,
+									     DB_FIELD (fields->data)->name,
+									     view, DB_FIELD (fields->data));
 					query_field_set_is_printed (nqf, TRUE);
 					query_add_field_before (qx->priv->q, nqf, qf);
 					fields = g_slist_next (fields);
@@ -230,8 +228,8 @@ prepare_query (QueryExec * qx)
 	while (list) {
 		QueryView *view = QUERY_VIEW (list->data);
 
-		if (IS_DB_TABLE (view->obj)) {
-			DbTable *table = DB_TABLE (view->obj);
+		if (IS_DB_TABLE (query_view_get_view_obj (view))) {
+			DbTable *table = DB_TABLE (query_view_get_view_obj (view));
 			gboolean all_pk_found = TRUE;
 			gboolean has_a_field = FALSE;
 			GSList *fields;
@@ -329,9 +327,9 @@ query_field_notify (QueryExec *qx, QueryField *qf)
 			g_free (mdn->descr);
 			mdn->descr = NULL;
 		}
-		if (mdn->widget) {
-			gtk_widget_destroy (mdn->widget);
-			mdn->widget = NULL;
+		if (mdn->data_entry) {
+			gtk_widget_destroy (mdn->data_entry);
+			mdn->data_entry = NULL;
 		}
 		qx->priv->missing_data = g_slist_remove (qx->priv->missing_data, mdn);
 		g_free (mdn);
@@ -357,9 +355,9 @@ dispose_missing_data (QueryExec *qx)
 			g_free (mdn->descr);
 			mdn->descr = NULL;
 		}
-		if (mdn->widget) {
-			gtk_widget_destroy (mdn->widget);
-			mdn->widget = NULL;
+		if (mdn->data_entry) {
+			gtk_widget_destroy (mdn->data_entry);
+			mdn->data_entry = NULL;
 		}
 		g_free (mdn);
 		list = g_slist_next (list);
@@ -373,15 +371,18 @@ dispose_missing_data (QueryExec *qx)
  * management of the opened widgets refering to the QX
  */
 
+static void managed_ui_dlg_destroyed (QueryExec *qx, GtkWidget *wid);
+static void managed_ui_ui_destroyed (QueryExec *qx, GtkWidget *wid);
 static void 
 add_managed_ui (QueryExec *qx, ExecUi *eui)
 {
 	qx->priv->exec_uis = g_slist_append (qx->priv->exec_uis, eui);
-	g_object_weak_ref (G_OBJECT (eui->dlg), (GWeakNotify) managed_ui_destroyed, qx);
+	g_object_weak_ref (G_OBJECT (eui->dlg), (GWeakNotify) managed_ui_dlg_destroyed, qx);
+	g_object_weak_ref (G_OBJECT (eui->ui), (GWeakNotify) managed_ui_ui_destroyed, qx);
 }
 
 static void 
-managed_ui_destroyed (QueryExec *qx, GtkWidget *wid)
+managed_ui_dlg_destroyed (QueryExec *qx, GtkWidget *wid)
 {
 	GSList *list;
 	ExecUi *eui = NULL;
@@ -393,10 +394,42 @@ managed_ui_destroyed (QueryExec *qx, GtkWidget *wid)
 		list = g_slist_next (list);
 	}
 
-	if (eui) { 
+	if (eui) {
+		if (eui->ui) {
+			g_object_weak_unref (G_OBJECT (eui->ui), (GWeakNotify) managed_ui_ui_destroyed, qx);
+			eui->ui = NULL;
+		}
+		qx->priv->exec_uis = g_slist_remove (qx->priv->exec_uis, eui);
+		g_free (eui);
+	}
+	else
+		g_warning ("EUI could not be found!");
+
+	if (!qx->priv->exec_uis && !qx->priv->disposed) /* time to destroy itself */
+		query_exec_free (qx);
+}
+
+static void 
+managed_ui_ui_destroyed (QueryExec *qx, GtkWidget *wid)
+{
+	GSList *list;
+	ExecUi *eui = NULL;
+
+	list = qx->priv->exec_uis;
+	while (list && !eui) {
+		if (EXEC_UI (list->data)->ui == wid)
+			eui = EXEC_UI (list->data);
+		list = g_slist_next (list);
+	}
+
+	if (eui) {
+		if (eui->dlg) {
+			g_object_weak_unref (G_OBJECT (eui->dlg), (GWeakNotify) managed_ui_dlg_destroyed, qx);
+			gtk_widget_destroy (GTK_WIDGET (eui->dlg));
+			eui->dlg = NULL;
+		}
 		qx->priv->exec_uis = g_slist_remove (qx->priv->exec_uis, eui);
 		g_free (eui);
-		g_print ("EUI removed!\n");
 	}
 	else
 		g_warning ("EUI could not be found!");
@@ -415,21 +448,18 @@ managed_ui_destroyed (QueryExec *qx, GtkWidget *wid)
  */
 
 
-static QueryExecStatus real_run_the_query (QueryExec * qx, ServerAccessQueryType qtype, gboolean show_query);
+static gboolean real_run_the_query (QueryExec * qx, ServerAccessQueryType qtype);
 
 /* The object asks for values before if applicable and runs the query,
    return TRUE if the query has been executed correctly */
-static QueryExecStatus
-query_exec_run (QueryExec * qx, gboolean show_query)
+static gboolean
+query_exec_run (QueryExec * qx)
 {
 	GSList *hold, *list, *missing_list = NULL;
-	QueryExecStatus retval;
+	gboolean retval = FALSE;
 
-	g_return_val_if_fail (qx && IS_QUERY_EXEC (qx), QUERY_EXEC_ERROR);
-	g_return_val_if_fail (qx->priv->q && IS_QUERY (qx->priv->q), QUERY_EXEC_ERROR);
-
-	/* just to tell we've been through this procedure */
-	qx->priv->run_done = TRUE;
+	g_return_val_if_fail (qx && IS_QUERY_EXEC (qx), FALSE);
+	g_return_val_if_fail (qx->priv->q && IS_QUERY (qx->priv->q), FALSE);
 
 	/* Rem: We assume that all the QueryField in the Query are used for the query
 	   (either in the FROM, in the WHERE part or in the joins part),
@@ -437,7 +467,7 @@ query_exec_run (QueryExec * qx, gboolean show_query)
 
 	/* if we have a SQL text only query, then run it, we don't need any values */
 	if (qx->priv->q->type == QUERY_TYPE_SQL) {
-		return real_run_the_query (qx, SERVER_ACCESS_QUERY_SQL, show_query);
+		return real_run_the_query (qx, SERVER_ACCESS_QUERY_SQL);
 	}
 
 	/* FIXME: if the query is not a SELECT query, then take care of it here */
@@ -446,21 +476,22 @@ query_exec_run (QueryExec * qx, gboolean show_query)
 	list = query_get_missing_value_fields (qx->priv->q);
 	hold = list;
 	while (list) {
+		DataHandler *dh;
 		QueryField *qf = QUERY_FIELD (list->data);
 		MissingDataNode *mdn = g_new0 (MissingDataNode, 1);
 
 		mdn->qf = qf;
 		g_object_weak_ref (G_OBJECT (qf), (GWeakNotify) query_field_notify, qx);
 		
-		mdn->descr = g_strdup_printf ("%s:", qf->name);
+		mdn->descr = g_strdup_printf ("%s:", query_field_get_name (qf));
 		
 		/* FIXME: in the next function, use the right 2nd argument in relation of the implementation
 		   of the QUERY_FIELD_VALUE QueryField  */
-		mdn->fns = server_access_get_object_display_fns (qx->priv->q->conf->srv, G_OBJECT (qf));
+		dh = server_access_get_object_handler (qx->priv->q->conf->srv, G_OBJECT (qf));
 		/* FIXME: use the default value defined by the QUERY_FIELD_VALUE QueryField 
 		   instead of ""*/
-		mdn->widget = GTK_WIDGET ((mdn->fns->widget_from_value) 
-					  (query_field_value_get_default_value (qf)));
+		mdn->data_entry = GTK_WIDGET (data_handler_get_widget_from_value (dh,
+						       (query_field_value_get_default_value (qf))));
 		missing_list = g_slist_append (missing_list, mdn);
 		list = g_slist_next (list);
 	}
@@ -468,8 +499,12 @@ query_exec_run (QueryExec * qx, gboolean show_query)
 		g_slist_free (hold);
 
 
-	if (missing_list == NULL)
-		return real_run_the_query (qx, SERVER_ACCESS_QUERY_SQL, show_query);
+	if (missing_list == NULL) {
+		retval = real_run_the_query (qx, SERVER_ACCESS_QUERY_SQL);
+
+		qx->priv->run_done = retval;
+		return retval;
+	}
 	else {
 		/* build the dialog to fill in values */
 		GtkWidget *dlg, *table, *label;
@@ -505,7 +540,7 @@ query_exec_run (QueryExec * qx, gboolean show_query)
 			gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i + 1,
 					  0, 0, 0, 0);
 			gtk_table_attach_defaults (GTK_TABLE (table),
-						   mdn->widget, 1, 2, i,
+						   mdn->data_entry, 1, 2, i,
 						   i + 1);
 			i++;
 			list = g_slist_next (list);
@@ -513,17 +548,19 @@ query_exec_run (QueryExec * qx, gboolean show_query)
 		gtk_widget_show_all (GTK_DIALOG (dlg)->vbox);
 		button = gtk_dialog_run (GTK_DIALOG (dlg));
 		qx->priv->missing_data = missing_list;
-		retval = QUERY_EXEC_CANCELLED;
+
 		switch (button) {
 		case GTK_RESPONSE_ACCEPT:
 			gtk_widget_hide (dlg); /* to avoid the destruction of the widgets in the MDN */
-			retval = real_run_the_query (qx, SERVER_ACCESS_QUERY_SQL, show_query);
+			retval = real_run_the_query (qx, SERVER_ACCESS_QUERY_SQL);
 			break;
 		default:
+			retval = FALSE;
 			break;
 		}
 		gtk_widget_destroy (dlg);
 
+		qx->priv->run_done = retval;
 		return retval;
 	}
 }
@@ -531,12 +568,11 @@ query_exec_run (QueryExec * qx, gboolean show_query)
 /* 
  * Now really run the query
  */
-static QueryExecStatus
-real_run_the_query (QueryExec * qx, ServerAccessQueryType qtype, gboolean show_query)
+static gboolean
+real_run_the_query (QueryExec * qx, ServerAccessQueryType qtype)
 {
 	GSList *hold, *mlist, *query_param_values = NULL;
 	QueryParamValue *qmv;
-	QueryExecStatus retval;
 
 	mlist = qx->priv->missing_data;
 	while (mlist) {
@@ -544,12 +580,12 @@ real_run_the_query (QueryExec * qx, ServerAccessQueryType qtype, gboolean show_q
 		GdaValue *value;
 
 		mdn = MISSING_DATA_NODE (mlist->data);
-		value = (mdn->fns->value_from_widget) (mdn->widget);
+		value = data_entry_get_gdavalue (DATA_ENTRY (mdn->data_entry));
 
 		/* the widgets are destroyed when the dialog is closed,
 		   so we set them to NULL here to avoid trying to destroy them
 		   again. */
-		mdn->widget = NULL;
+		mdn->data_entry = NULL;
 
 		qmv = g_new0 (QueryParamValue, 1);
 		qmv->field = mdn->qf;
@@ -573,44 +609,10 @@ real_run_the_query (QueryExec * qx, ServerAccessQueryType qtype, gboolean show_q
 		g_slist_free_1 (hold);
 	}
 
-	/* show the query and ask if the user wants to proceed */
-	if (show_query) {
-		GtkWidget *dlg, *label;
-		gchar *str;
-		gint button;
-		dlg = gtk_dialog_new_with_buttons (_("Confirmation for query execution"), NULL, 0,
-						   GTK_STOCK_OK, GTK_RESPONSE_OK,
-						   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-						   NULL);
-		str = g_strdup_printf (_("Run the following query?\n\n%s"), qx->priv->executed_query);
-		label = gtk_label_new (str);
-		g_free (str);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), label, TRUE, TRUE, GNOME_PAD);
-		gtk_widget_show (label);
-		button = gtk_dialog_run (GTK_DIALOG (dlg));
-		switch (button) {
-		case GTK_RESPONSE_OK:
-			qx->priv->sres = server_access_do_query (qx->priv->q->conf->srv, qx->priv->executed_query, qtype);
-			if (qx->priv->sres)
-				retval = QUERY_EXEC_OK;
-			else
-				retval = QUERY_EXEC_ERROR;
-			break;
-		default:
-			retval = QUERY_EXEC_CANCELLED;
-			break;
-		}
-		gtk_widget_destroy (dlg);
-	}
-	else {
-		qx->priv->sres = server_access_do_query (qx->priv->q->conf->srv, qx->priv->executed_query, qtype);
-		if (qx->priv->sres)
-			retval = QUERY_EXEC_OK;
-		else
-			retval = QUERY_EXEC_ERROR;
-	}
-	
-	return retval;
+	/* run the query */
+	qx->priv->sres = server_access_do_query (qx->priv->q->conf->srv, qx->priv->executed_query, qtype);
+
+	return qx->priv->sres ? TRUE : FALSE;
 }
 
 
@@ -634,11 +636,12 @@ static GtkWidget *prepare_ui_dialog (QueryExec *qx, gchar *title);
 static GtkWidget *get_real_edit_form (QueryExec * qx, gulong numrow, gboolean empty);
 static void form_contents_modified_cb (DataForm *form, ExecUi *eui);
 GtkWidget *
-query_exec_get_dialog_edit_form (QueryExec * qx, gulong numrow)
+query_exec_get_edit_form (QueryExec * qx, gulong numrow)
 {
-	GtkWidget *dlg, *newui;
+	GtkWidget *dlg = NULL, *newui;
 	ExecUi *eui;
 	gchar *str;
+	gboolean ok = TRUE;
 
 	g_return_val_if_fail (qx && IS_QUERY_EXEC (qx), NULL);
 	g_return_val_if_fail (qx->priv->q && IS_QUERY (qx->priv->q), NULL);
@@ -648,25 +651,27 @@ query_exec_get_dialog_edit_form (QueryExec * qx, gulong numrow)
 
 	/* init RS if not done */
 	if (!qx->priv->run_done)
-		query_exec_run (qx, TRUE);
+		ok = query_exec_run (qx);
 
-	/* dialog preparation */
-	str = g_strdup_printf (_("Recordset Edition (query '%s')"),
-			       qx->priv->q->name);
-	dlg = prepare_ui_dialog (qx, str);
-	g_free (str);
-	eui->dlg = dlg;
-
-	newui = get_real_edit_form (qx, numrow, FALSE);
-	eui->ui = newui;
-	g_signal_connect (G_OBJECT (newui), "contents_modified",
-			  G_CALLBACK (form_contents_modified_cb), eui);
-
-	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox), newui);
-	gtk_widget_show_all (newui);
-
-	/* addtion to the list of managed dialogs */
-	add_managed_ui (qx, eui);
+	if (ok) {
+		/* dialog preparation */
+		str = g_strdup_printf (_("Recordset Edition (query '%s')"),
+				       qx->priv->q->name);
+		dlg = prepare_ui_dialog (qx, str);
+		g_free (str);
+		eui->dlg = dlg;
+		
+		newui = get_real_edit_form (qx, numrow, FALSE);
+		eui->ui = newui;
+		g_signal_connect (G_OBJECT (newui), "contents_modified",
+				  G_CALLBACK (form_contents_modified_cb), eui);
+		
+		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox), newui);
+		gtk_widget_show (newui);
+		
+		/* addtion to the list of managed dialogs */
+		add_managed_ui (qx, eui);
+	}
 
 	return dlg;
 }
@@ -688,9 +693,10 @@ form_contents_modified_cb (DataForm *form, ExecUi *eui)
 GtkWidget *
 query_exec_get_insert_form (QueryExec * qx)
 {
-	GtkWidget *dlg, *newui;
+	GtkWidget *dlg = NULL, *newui;
 	ExecUi *eui;
 	gchar *str;
+	gboolean ok = TRUE;
 
 	g_return_val_if_fail (qx && IS_QUERY_EXEC (qx), NULL);
 	g_return_val_if_fail (qx->priv->q && IS_QUERY (qx->priv->q), NULL);
@@ -700,23 +706,25 @@ query_exec_get_insert_form (QueryExec * qx)
 
 	/* init RS if not done */
 	if (!qx->priv->run_done)
-		query_exec_run (qx, TRUE);
+		ok = query_exec_run (qx);
 
-	/* dialog preparation */
-	str = g_strdup_printf (_("Data insertion in '%s' (query '%s')"),
-			       qx->priv->modif_table->name, qx->priv->q->name);
-	dlg = prepare_ui_dialog (qx, str);
-	g_free (str);
-	eui->dlg = dlg;
-
-	newui = get_real_edit_form (qx, 0, TRUE);
-	eui->ui = newui;
-
-	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox), newui);
-	gtk_widget_show_all (newui);
-
-	/* addtion to the list of managed dialogs */
-	add_managed_ui (qx, eui);
+	if (ok) {
+		/* dialog preparation */
+		str = g_strdup_printf (_("Data insertion in '%s' (query '%s')"),
+				       qx->priv->modif_table->name, qx->priv->q->name);
+		dlg = prepare_ui_dialog (qx, str);
+		g_free (str);
+		eui->dlg = dlg;
+		
+		newui = get_real_edit_form (qx, 0, TRUE);
+		eui->ui = newui;
+		
+		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox), newui);
+		gtk_widget_show (newui);
+		
+		/* addtion to the list of managed dialogs */
+		add_managed_ui (qx, eui);
+	}
 
 	return dlg;
 }
@@ -774,9 +782,10 @@ static GtkWidget * get_real_consult_grid (QueryExec * qx, gulong numrow);
 GtkWidget *
 query_exec_get_consult_grid   (QueryExec * exec, gulong numrow)
 {
-	GtkWidget *dlg, *newui;
+	GtkWidget *dlg = NULL, *newui;
 	ExecUi *eui;
 	gchar *str;
+	gboolean ok = TRUE;
 
 	g_return_val_if_fail (exec && IS_QUERY_EXEC (exec), NULL);
 	g_return_val_if_fail (exec->priv->q && IS_QUERY (exec->priv->q), NULL);
@@ -786,23 +795,25 @@ query_exec_get_consult_grid   (QueryExec * exec, gulong numrow)
 
 	/* init RS if not done */
 	if (!exec->priv->run_done)
-		query_exec_run (exec, TRUE);
+		ok = query_exec_run (exec);
 
-	/* dialog preparation */
-	str = g_strdup_printf (_("Recordset tabular view (query '%s')"),
-			       exec->priv->q->name);
-	dlg = prepare_ui_dialog (exec, str);
-	g_free (str);
-	eui->dlg = dlg;
-
-	newui = get_real_consult_grid (exec, numrow);
-	eui->ui = newui;
-
-	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox), newui);
-	gtk_widget_show_all (newui);
-
-	/* addtion to the list of managed dialogs */
-	add_managed_ui (exec, eui);
+	if (ok) {
+		/* dialog preparation */
+		str = g_strdup_printf (_("Recordset tabular view (query '%s')"),
+				       exec->priv->q->name);
+		dlg = prepare_ui_dialog (exec, str);
+		g_free (str);
+		eui->dlg = dlg;
+		
+		newui = get_real_consult_grid (exec, numrow);
+		eui->ui = newui;
+		
+		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox), newui);
+		gtk_widget_show (newui);
+		
+		/* addtion to the list of managed dialogs */
+		add_managed_ui (exec, eui);
+	}
 
 	return dlg;	
 }
@@ -873,7 +884,8 @@ sub_action_insert (GtkWidget * wid, QueryExec * qx)
 	GtkWidget *dlg;
 
 	dlg = query_exec_get_insert_form (qx);
-	gtk_widget_show (dlg);
+	if (dlg)
+		gtk_widget_show (dlg);
 }
 
 static void
@@ -881,7 +893,7 @@ sub_action_edit (GtkWidget * wid, gulong row, QueryExec * qx)
 {
 	GtkWidget *dlg;
 
-	dlg = query_exec_get_dialog_edit_form (qx, row);
+	dlg = query_exec_get_edit_form (qx, row);
 	gtk_widget_show (dlg);
 }
 
@@ -926,21 +938,23 @@ sub_action_viewall (GtkWidget * wid, QueryExec * qx)
 	if (found)
 		gdk_window_raise (GTK_WIDGET (eui->dlg)->window);
 	else {
-		str = g_strdup_printf (_("Execution of query '%s'"), qx->priv->q->name);
-		dlg = prepare_ui_dialog (qx, str);
-		g_free (str);
-
-		eui = g_new0 (ExecUi, 1);
-		eui->qx = qx;
-		eui->dlg = dlg;
 		newui = query_exec_get_consult_grid (qx, 0);
-		eui->ui = newui;
-
-		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox), newui);
-		gtk_widget_show (newui);
-
-		gtk_widget_show (dlg);
-		add_managed_ui (qx, eui);
+		if (newui) {
+			str = g_strdup_printf (_("Execution of query '%s'"), qx->priv->q->name);
+			dlg = prepare_ui_dialog (qx, str);
+			g_free (str);
+			
+			eui = g_new0 (ExecUi, 1);
+			eui->qx = qx;
+			eui->dlg = dlg;
+			eui->ui = newui;
+			
+			gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox), newui);
+			gtk_widget_show (newui);
+			
+			gtk_widget_show (dlg);
+			add_managed_ui (qx, eui);
+		}
 	}
 }
 
