@@ -376,15 +376,12 @@ table_edit_initialize (TableEdit * wid)
 				g_strdup_printf ("nextval('%s')", seq->name);
 		}
 		else {
-			if (DB_FIELD (list->data)->default_val)
-				text[5] =
-					g_strdup (DB_FIELD (list->data)->
-						  default_val);
+			if (DB_FIELD (list->data)->default_value)
+				text[5] = gda_value_stringify (DB_FIELD (list->data)->default_value);
 		}
 		if (!text[5])
 			text[5] = g_strdup ("");
-		row = gtk_clist_append (GTK_CLIST (wid->clist_table_fields),
-					text);
+		row = gtk_clist_append (GTK_CLIST (wid->clist_table_fields), text);
 		g_free (text[2]);
 		g_free (text[5]);
 		/* DbField pointers are attached to the rows */
@@ -671,6 +668,7 @@ add_field_dialog_cb (GnomeDialog * wid, gint button_number, gpointer * data)
 	ConfManager *conf;
 	GString *str;
 	ServerDataType *stype;
+	ServerResultset *rs;
 
 	switch (button_number) {
 	case 0:		/* OK button */
@@ -704,9 +702,12 @@ add_field_dialog_cb (GnomeDialog * wid, gint button_number, gpointer * data)
 							      (name)),
 					  stype->sqlname);
 		/* now do the query */
-		server_access_do_query (conf->srv, str->str, SERVER_ACCESS_QUERY_SQL);
+		rs = server_access_do_query (conf->srv, str->str, SERVER_ACCESS_QUERY_SQL);
 		g_string_free (str, TRUE);
-		database_refresh (conf->db, conf->srv);
+		if (rs) {
+			g_object_unref (G_OBJECT (rs));
+			database_refresh (conf->db, conf->srv);
+		}
 		break;
 	case 1:		/* Cancel button */
 		break;
@@ -848,8 +849,8 @@ field_created_cb (Database * db, DbField * field, gpointer data)
 		text[5] = g_strdup_printf ("nextval('%s')", seq->name);
 	}
 	else {
-		if (field->default_val)
-			text[5] = g_strdup (field->default_val);
+		if (field->default_value)
+			text[5] = gda_value_stringify (field->default_value);
 	}
 	if (!text[5])
 		text[5] = g_strdup ("");
@@ -976,8 +977,8 @@ sequence_combo_box_cb (GtkWidget * wid, gpointer data)
 			str = g_strdup_printf ("nextval('%s')", seq->name);
 		}
 		else {
-			if (te->field->default_val)
-				str = g_strdup (te->field->default_val);
+			if (te->field->default_value)
+				str = gda_value_stringify (te->field->default_value);
 		}
 		if (!str)
 			str = g_strdup ("");
